package com.microsoft.azure.management.datalake.store;

import com.microsoft.azure.CloudException;
import com.microsoft.azure.management.datalake.store.models.AclStatusResult;
import com.microsoft.azure.management.datalake.store.models.DataLakeStoreAccount;
import com.microsoft.azure.management.datalake.store.models.FileOperationResult;
import com.microsoft.azure.management.datalake.store.models.FileStatusProperties;
import com.microsoft.azure.management.datalake.store.models.FileStatusResult;
import com.microsoft.azure.management.datalake.store.models.FileStatusesResult;
import com.microsoft.azure.management.datalake.store.models.FileType;
import com.microsoft.azure.management.resources.models.ResourceGroup;
import org.apache.commons.lang3.StringUtils;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import sun.misc.IOUtils;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class DataLakeStoreFilesystemOperationsTests extends DataLakeStoreManagementTestBase {
    // constants
    private static String folderToCreate = "SDKTestFolder01";
    private static String fileToCreateWithContents = "SDKTestFile02.txt";
    private static String fileToCopy = "SDKTestCopyFile01.txt";
    private static String fileToConcatTo = "SDKTestConcatFile01.txt";

    private static String fileContentsToAdd = "These are some random test contents 1234!@";

    private static String rgName = generateName("javaadlsrg");
    private static String adlsAcct = generateName("javaadlsacct");


    @BeforeClass
    public static void setup() throws Exception {
        createClients();
        ResourceGroup group = new ResourceGroup();
        String location = "eastus2";
        group.setLocation(location);
        resourceManagementClient.getResourceGroupsOperations().createOrUpdate(rgName, group);

        // create storage and ADLS accounts, setting the accessKey
        DataLakeStoreAccount adlsAccount = new DataLakeStoreAccount();
        adlsAccount.setLocation(location);
        adlsAccount.setName(adlsAcct);
        dataLakeStoreAccountManagementClient.getAccountOperations().create(rgName, adlsAcct, adlsAccount);
    }

    @AfterClass
    public static void cleanup() throws Exception {
        try {
            resourceManagementClient.getResourceGroupsOperations().delete(rgName);
        }
        catch (Exception e) {
            // ignore failures during cleanup, as it is best effort
        }
    }

    // tests
    @Test
    public void DataLakeStoreFileSystemFolderCreate() throws Exception
    {
        String folderPath = CreateFolder(adlsAcct, true);
        GetAndCompareFileOrFolder(adlsAcct, folderPath,
                FileType.DIRECTORY, 0);
    }

    /*
    TODO: Re-enable code when Expiry is live on the server again
    @Test
    public void DataLakeStoreFileSystemSetAndRemoveExpiry()
    {
        const long maxTimeInMilliseconds = 253402300800000;
        var filePath = CreateFile(adlsAcct, false, true);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE, 0);

        // verify it does not have an expiration
        var fileInfo = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().GetFileInfo(adlsAcct, filePath);
        Assert.assertTrue(fileInfo.FileInfo.ExpirationTime <= 0 || fileInfo.FileInfo.ExpirationTime == maxTimeInMilliseconds, "Expiration time was not equal to 0 or DateTime.MaxValue.Ticks! Actual value reported: " + fileInfo.FileInfo.ExpirationTime);

        // set the expiration time as an absolute value

        var toSetAbsolute = ToUnixTimeStampMs(HttpMockServer.GetVariable("absoluteTime", DateTime.Now.AddSeconds(120).ToString()));
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetFileExpiry(adlsAcct, filePath, ExpiryOptionType.Absolute, toSetAbsolute);
        fileInfo = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().GetFileInfo(adlsAcct, filePath);
        VerifyTimeInAcceptableRange(toSetAbsolute, fileInfo.FileInfo.ExpirationTime.Value);

        // set the expiration time relative to now
        var toSetRelativeToNow = ToUnixTimeStampMs(HttpMockServer.GetVariable("relativeTime", DateTime.Now.AddSeconds(120).ToString()));
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetFileExpiry(adlsAcct, filePath, ExpiryOptionType.RelativeToNow, 120 * 1000);
        fileInfo = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().GetFileInfo(adlsAcct, filePath);
        VerifyTimeInAcceptableRange(toSetRelativeToNow, fileInfo.FileInfo.ExpirationTime.Value);

        // set expiration time relative to the creation time
        var toSetRelativeCreationTime = fileInfo.FileInfo.CreationTime.Value + (120 * 1000);
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetFileExpiry(adlsAcct, filePath, ExpiryOptionType.RelativeToCreationDate, 120 * 1000);
        fileInfo = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().GetFileInfo(adlsAcct, filePath);
        VerifyTimeInAcceptableRange(toSetRelativeCreationTime, fileInfo.FileInfo.ExpirationTime.Value);

        // reset expiration time to never
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetFileExpiry(adlsAcct, filePath, ExpiryOptionType.NeverExpire);
        fileInfo = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().GetFileInfo(adlsAcct, filePath);
        Assert.assertTrue(fileInfo.FileInfo.ExpirationTime <= 0 || fileInfo.FileInfo.ExpirationTime == maxTimeInMilliseconds, "Expiration time was not equal to 0 or DateTime.MaxValue.Ticks! Actual value reported: " + fileInfo.FileInfo.ExpirationTime);
    }

    @Test
    public void DataLakeStoreFileSystemNegativeExpiry()
    {
        const long maxTimeInMilliseconds = 253402300800000;
        var filePath = CreateFile(adlsAcct, false, true);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE, 0);

        // verify it does not have an expiration
        var fileInfo = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().GetFileInfo(adlsAcct, filePath);
        Assert.assertTrue(fileInfo.FileInfo.ExpirationTime <= 0 || fileInfo.FileInfo.ExpirationTime == maxTimeInMilliseconds, "Expiration time was not equal to 0 or DateTime.MaxValue.Ticks! Actual value reported: " + fileInfo.FileInfo.ExpirationTime);

        // set the expiration time as an absolute value that is less than the creation time
        var toSetAbsolute = ToUnixTimeStampMs(HttpMockServer.GetVariable("absoluteNegativeTime", DateTime.Now.AddSeconds(-120).ToString()));
        Assert.assertThrows<CloudException>(() => dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetFileExpiry(adlsAcct, filePath, ExpiryOptionType.Absolute, toSetAbsolute));

        // set the expiration time as an absolute value that is greater than max allowed time
        toSetAbsolute = ToUnixTimeStampMs(DateTime.MaxValue.ToString()) + 1000;
        Assert.assertThrows<CloudException>(() => dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetFileExpiry(adlsAcct, filePath, ExpiryOptionType.Absolute, toSetAbsolute));

        // reset expiration time to never with a value and confirm the value is not honored
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetFileExpiry(adlsAcct, filePath, ExpiryOptionType.NeverExpire, 400);
        fileInfo = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().GetFileInfo(adlsAcct, filePath);
        Assert.assertTrue(fileInfo.FileInfo.ExpirationTime <= 0 || fileInfo.FileInfo.ExpirationTime == maxTimeInMilliseconds, "Expiration time was not equal to 0 or DateTime.MaxValue.Ticks! Actual value reported: " + fileInfo.FileInfo.ExpirationTime);
    }
    */

    @Test
    public void DataLakeStoreFileSystemListFolderContents() throws Exception
    {
        String folderPath = CreateFolder(adlsAcct, true);
        GetAndCompareFileOrFolder(adlsAcct, folderPath,
                FileType.DIRECTORY, 0);

        String filePath = CreateFile(adlsAcct, false, true, folderPath);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE, 0);

        // List all the contents in the folder
        FileStatusesResult listFolderResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().listFileStatus(adlsAcct, folderPath).getBody();

        // We know that this directory is brand new, so the contents should only be the one file.
        Assert.assertEquals(1, listFolderResponse.getFileStatuses().getFileStatus().size());
        Assert.assertEquals(FileType.FILE, listFolderResponse.getFileStatuses().getFileStatus().get(0).getType());
    }

    @Test
    public void DataLakeStoreFileSystemEmptyFileCreate() throws Exception
    {
        String filePath = CreateFile(adlsAcct, false, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE, 0);
    }

    @Test
    public void DataLakeStoreFileSystemFileCreateWithContents() throws Exception
    {
        String filePath = CreateFile(adlsAcct, true, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE,
                fileContentsToAdd.length());
        CompareFileContents(adlsAcct, filePath,
                fileContentsToAdd);
    }

    @Test
    public void DataLakeStoreFileSystemAppendToFile() throws Exception
    {
        String filePath = CreateFile(adlsAcct, false, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE, 0);

        // Append to the file that we created
        String fileContentsToAppend = "More test contents, that were appended!";
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().append(adlsAcct, filePath, fileContentsToAppend.getBytes());

        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE,
                fileContentsToAppend.length());
    }

    @Test
    public void DataLakeStoreFileSystemConcatenateFiles() throws Exception
    {
        String filePath1 = CreateFile(adlsAcct, true, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath1, FileType.FILE,
                fileContentsToAdd.length());

        String filePath2 = CreateFile(adlsAcct, true, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath2, FileType.FILE,
                fileContentsToAdd.length());

        String targetFolder = CreateFolder(adlsAcct, true);

        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().concat(
                adlsAcct,
                String.format("%s/%s", targetFolder, fileToConcatTo),
                Arrays.asList(new String[]{filePath1, filePath2})
        );

        GetAndCompareFileOrFolder(adlsAcct,
                String.format("%s/%s", targetFolder, fileToConcatTo),
                FileType.FILE,
                fileContentsToAdd.length() * 2);

        // Attempt to get the files that were concatted together, which should fail and throw
        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath1);
            Assert.assertTrue("Able to get the old file after concat", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }

        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath2);
            Assert.assertTrue("Able to get the old file after concat", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }
    }

    @Test
    public void DataLakeStoreFileSystemMsConcatenateFiles() throws Exception
    {
        String filePath1 = CreateFile(adlsAcct, true, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath1, FileType.FILE,
                fileContentsToAdd.length());

        String filePath2 = CreateFile(adlsAcct, true, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath2, FileType.FILE,
                fileContentsToAdd.length());

        String targetFolder = CreateFolder(adlsAcct, true);

        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().msConcat(
                adlsAcct,
                String.format("%s/%s", targetFolder, fileToConcatTo),
                String.format("sources=%s,%s", filePath1, filePath2).getBytes(),
                false);

        GetAndCompareFileOrFolder(adlsAcct,
                String.format("%s/%s", targetFolder, fileToConcatTo),
                FileType.FILE,
                fileContentsToAdd.length() * 2);

        // Attempt to get the files that were concatted together, which should fail and throw
        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath1);
            Assert.assertTrue("Able to get the old file after concat", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }

        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath2);
            Assert.assertTrue("Able to get the old file after concat", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }
    }

    @Test
    public void DataLakeStoreFileSystemMsConcatDeleteDir() throws Exception
    {
        String concatFolderPath = String.format("%s/%s", folderToCreate,
                "msconcatFolder");
        String filePath1 = CreateFile(adlsAcct, true, true,
                concatFolderPath);
        GetAndCompareFileOrFolder(adlsAcct, filePath1, FileType.FILE,
                fileContentsToAdd.length());

        String filePath2 = CreateFile(adlsAcct, true, true,
                concatFolderPath);
        GetAndCompareFileOrFolder(adlsAcct, filePath2, FileType.FILE,
                fileContentsToAdd.length());

        String targetFolder = CreateFolder(adlsAcct, true);

        String destination = String.format("%s/%s", targetFolder, fileToConcatTo);

        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().msConcat(
                adlsAcct,
                destination,
                String.format("sources=%s,%s", filePath1, filePath2).getBytes(),
                true);

        GetAndCompareFileOrFolder(adlsAcct,
                String.format("%s/%s", targetFolder, fileToConcatTo),
                FileType.FILE,
                fileContentsToAdd.length()*2);

        // Attempt to get the files that were concatted together, which should fail and throw
        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath1);
            Assert.assertTrue("Able to get the old file after concat", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }

        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath2);
            Assert.assertTrue("Able to get the old file after concat", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }

        // Attempt to get the folder that was created for concat, which should fail and be deleted.
        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, concatFolderPath);
            Assert.assertTrue("Able to get the old folder after concat", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }
    }

    @Test
    public void DataLakeStoreFileSystemMoveFileAndFolder() throws Exception
    {
        String filePath = CreateFile(adlsAcct, true, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE,
                fileContentsToAdd.length());

        String targetFolder1 = CreateFolder(adlsAcct, true);
        String folderToMove = "SDKTestMoveFolder01";
        String targetFolder2 = generateName(folderToMove);

        // Move file first
        String fileToMove = "SDKTestMoveFile01.txt";
        FileOperationResult moveFileResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().rename(
                adlsAcct,
                filePath,
                String.format("%s/%s", targetFolder1, fileToMove)).getBody();
        Assert.assertTrue(moveFileResponse.getOperationResult());
        GetAndCompareFileOrFolder(adlsAcct,
                String.format("%s/%s", targetFolder1, fileToMove),
                FileType.FILE,
                fileContentsToAdd.length());

        // Ensure the old file is gone
        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath);
            Assert.assertTrue("Able to get the old file after rename", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }

        // Now move folder completely.
        FileOperationResult moveFolderResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().rename(
                adlsAcct, 
                targetFolder1,
                targetFolder2).getBody();
        Assert.assertTrue(moveFolderResponse.getOperationResult());

        GetAndCompareFileOrFolder(adlsAcct, targetFolder2,
                FileType.DIRECTORY, 0);

        // ensure all the contents of the folder moved
        // List all the contents in the folder
        FileStatusesResult listFolderResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().listFileStatus(
                adlsAcct, 
                targetFolder2).getBody();

        // We know that this directory is brand new, so the contents should only be the one file.
        Assert.assertEquals(1, listFolderResponse.getFileStatuses().getFileStatus().size());
        Assert.assertEquals(FileType.FILE, listFolderResponse.getFileStatuses().getFileStatus().get(0).getType());

        try {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, targetFolder1);
            Assert.assertTrue("Able to get the old folder after rename", false);
        }
        catch (Exception e) {
            Assert.assertTrue(e instanceof CloudException);
        }
    }

    @Test
    public void DataLakeStoreFileSystemDeleteFolder() throws Exception
    {
        String folderPath = CreateFolder(adlsAcct, true);
        GetAndCompareFileOrFolder(adlsAcct, folderPath,
                FileType.DIRECTORY, 0);
        DeleteFolder(adlsAcct, folderPath, true, false);
        //WORK AROUND: Bug 4717659 makes it so even empty folders have contents.

        // delete again expecting failure.
        DeleteFolder(adlsAcct, folderPath, false, true);

        // delete a folder with contents
        String filePath = CreateFile(adlsAcct, true, true, folderPath);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE,
                fileContentsToAdd.length());

        // should fail if recurse is not set
        DeleteFolder(adlsAcct, folderPath, false, true);

        // Now actually delete
        DeleteFolder(adlsAcct, folderPath, true, false);

        // delete again expecting failure.
        DeleteFolder(adlsAcct, folderPath, true, true);
    }

    @Test
    public void DataLakeStoreFileSystemDeleteFile() throws Exception
    {
        String filePath = CreateFile(adlsAcct, true, true, folderToCreate);
        GetAndCompareFileOrFolder(adlsAcct, filePath, FileType.FILE,
                fileContentsToAdd.length());
        DeleteFile(adlsAcct, filePath, false);

        // try to delete it again, which should fail
        DeleteFile(adlsAcct, filePath, true);
    }

    @Test
    public void DataLakeStoreFileSystemGetAndSetAcl() throws Exception
    {
        AclStatusResult currentAcl = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody();

        List<String> aclToReplaceWith = new ArrayList<String>(currentAcl.getAclStatus().getEntries());
        String originalOther = "";
        String toReplace = "other::rwx";
        for (int i = 0; i < aclToReplaceWith.size(); i++)
        {
            if (aclToReplaceWith.get(i).startsWith("other"))
            {
                originalOther = aclToReplaceWith.get(i);
                aclToReplaceWith.set(i, toReplace);
                break;
            }
        }

        Assert.assertFalse(originalOther == null || StringUtils.isEmpty(originalOther));

        // Set the other acl to RWX
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().setAcl(adlsAcct, "/",
                StringUtils.join(aclToReplaceWith, ","));

        AclStatusResult newAcl = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody();
        // verify the ACL actually changed

        // Check the access first and assert that it returns OK (note: this is currently only for the user making the request, so it is not testing "other")
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().checkAccess(
                adlsAcct, 
                "/",
                "rwx");

        boolean foundIt = false;
        for (String entry: newAcl.getAclStatus().getEntries())
        {
            if(entry.startsWith("other")) {
                Assert.assertEquals(toReplace, entry);
                foundIt = true;
                break;
            }
        }

        Assert.assertTrue(foundIt);

        // Set it back using specific entry
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().modifyAclEntries(
                adlsAcct, 
                "/",
                originalOther);

        // Now confirm that it equals the original ACL
        List<String> finalEntries = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody()
                .getAclStatus().getEntries();
        for (String entry: finalEntries)
        {
            boolean found = false;
            for(String curEntry: currentAcl.getAclStatus().getEntries()) {
                if(curEntry.toUpperCase().equals(entry.toUpperCase())) {
                    found = true;
                    break;
                }
            }

            Assert.assertTrue(found);
        }

        Assert.assertEquals(finalEntries.size(), currentAcl.getAclStatus().getEntries().size());
    }

    @Test
    public void DataLakeStoreFileSystemSetFileProperties() throws Exception
    {
        // This test simply tests that all bool/empty return actions return successfully

        String filePath = CreateFile(adlsAcct, true, true, folderToCreate);
        FileStatusProperties originalFileStatus =
                dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath).getBody().getFileStatus();
        // TODO: Set replication on file, this has been removed until it is confirmed as a supported API.
            /*
            var replicationResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetReplication(adlsAcct, filePath, 3);
            Assert.assertTrue(replicationResponse.Boolean);
            */

            /*
         * This API is available but all values put into it are ignored. Commenting this out until this API is fully functional.
        Assert.assertEquals(3,
            dataLakeFileSystemClient.FileSystem.getFileStatus(filePath)
                .FileStatus.Replication);
        */

        // set the time on the file
        // We use a static date for now since we aren't interested in whether the value is set properly, only that the method returns a 200.
            /* TODO: Re enable once supported.
            var timeToSet = new DateTime(2015, 10, 26, 14, 30, 0).Ticks;
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().SetTimes(adlsAcct, filePath, timeToSet, timeToSet);

            var fileStatusAfterTime =
                dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(adlsAcct, filePath).FileStatus;
            */

            /*
         * This API is available but all values put into it are ignored. Commenting this out until this API is fully functional.
        Assert.assertTrue(
            fileStatusAfterTime.ModificationTime == timeToSet && fileStatusAfterTime.AccessTime == timeToSet);
        */

        // TODO: Symlink creation is explicitly not supported, but when it is this should be enabled.
            /*
            var symLinkName = generateName("testPath/symlinktest1");
            Assert.assertThrows<CloudException>(() => dataLakeStoreFileSystemManagementClient.getFileSystemOperations().CreateSymLink(adlsAcct, filePath, symLinkName, true));
            */

        // Once symlinks are available, remove the throws test and uncomment out this code.
        // Assert.assertTrue(createSymLinkResponse.StatusCode == HttpStatusCode.OK);
        // Assert.assertDoesNotThrow(() => dataLakeFileSystemClient.FileSystem.getFileStatus(symLinkName));
    }

    @Test
    public void DataLakeStoreFileSystemGetAcl() throws Exception
    {
        AclStatusResult aclGetResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody();

        Assert.assertNotNull(aclGetResponse.getAclStatus());
        Assert.assertTrue(aclGetResponse.getAclStatus().getEntries().size() > 0);
        Assert.assertTrue(aclGetResponse.getAclStatus().getOwner() != null && StringUtils.isNotEmpty(aclGetResponse.getAclStatus().getOwner()));
        Assert.assertTrue(aclGetResponse.getAclStatus().getGroup() != null && StringUtils.isNotEmpty(aclGetResponse.getAclStatus().getGroup()));
    }

    @Test
    public void DataLakeStoreFileSystemSetAcl() throws Exception
    {
        AclStatusResult aclGetResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody();

        Assert.assertNotNull(aclGetResponse.getAclStatus());
        Assert.assertTrue(aclGetResponse.getAclStatus().getEntries().size() > 0);

        int currentCount = aclGetResponse.getAclStatus().getEntries().size();

        // add an entry to the ACL Entries
        String newAcls = StringUtils.join(aclGetResponse.getAclStatus().getEntries(), ",");
        String aclUserId = UUID.randomUUID().toString();
        newAcls += String.format(",user:%s:rwx", aclUserId);

        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().setAcl(adlsAcct, 
                "/",
                newAcls);

        // retrieve the ACL again and confirm the new entry is present
        aclGetResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody();

        Assert.assertNotNull(aclGetResponse.getAclStatus());
        Assert.assertTrue(aclGetResponse.getAclStatus().getEntries().size() > 0);
        Assert.assertEquals(currentCount + 1, aclGetResponse.getAclStatus().getEntries().size());

        boolean found = false;
        for (String entry:  aclGetResponse.getAclStatus().getEntries()) {
            if(entry.contains(aclUserId)) {
                found = true;
                break;
            }
        }

        Assert.assertTrue(found);
    }

    @Test
    public void DataLakeStoreFileSystemSetDeleteAclEntry() throws Exception
    {
        AclStatusResult aclGetResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody();

        Assert.assertNotNull(aclGetResponse.getAclStatus());
        Assert.assertTrue(aclGetResponse.getAclStatus().getEntries().size() > 0);

        int currentCount = aclGetResponse.getAclStatus().getEntries().size();
        // add an entry to the ACL Entries
        String aclUserId = UUID.randomUUID().toString();
        String newAce = String.format("user:%s:rwx", aclUserId);

        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().modifyAclEntries(adlsAcct, "",
                newAce);

        // retrieve the ACL again and confirm the new entry is present
        aclGetResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody();

        Assert.assertNotNull(aclGetResponse.getAclStatus());
        Assert.assertTrue(aclGetResponse.getAclStatus().getEntries().size() > 0);
        Assert.assertEquals(currentCount + 1, aclGetResponse.getAclStatus().getEntries().size());

        boolean found = false;
        for (String entry:  aclGetResponse.getAclStatus().getEntries()) {
            if(entry.contains(aclUserId)) {
                found = true;
                break;
            }
        }

        Assert.assertTrue(found);

        // now remove the entry
        String aceToRemove = String.format(",user:%s", aclUserId);
        dataLakeStoreFileSystemManagementClient.getFileSystemOperations().removeAclEntries(
                adlsAcct, 
                "/",
                aceToRemove);

        // retrieve the ACL again and confirm the new entry is present
        aclGetResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getAclStatus(adlsAcct, "/").getBody();

        Assert.assertNotNull(aclGetResponse.getAclStatus());
        Assert.assertTrue(aclGetResponse.getAclStatus().getEntries().size() > 0);
        Assert.assertEquals(currentCount, aclGetResponse.getAclStatus().getEntries().size());

        found = false;
        for (String entry:  aclGetResponse.getAclStatus().getEntries()) {
            if(entry.contains(aclUserId)) {
                found = true;
                break;
            }
        }

        Assert.assertFalse(found);
    }
    
    // helper methods
    private String CreateFolder(String caboAccountName, boolean randomName) throws Exception
    {
        // Create a folder
        String folderPath = randomName
                ? generateName(folderToCreate)
                : folderToCreate;

        FileOperationResult response = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().mkdirs(caboAccountName, folderPath).getBody();
        Assert.assertTrue(response.getOperationResult());

        return folderPath;
    }

    private String CreateFile(String caboAccountName, boolean withContents, boolean randomName, String folderName) throws Exception
    {
        String fileToCreate = "SDKTestFile01.txt";
        String filePath = randomName ? generateName(String.format("%s/%s", folderName, fileToCreate)) : String.format("%s/%s", folderName, fileToCreate);

        if (!withContents)
        {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().create(
                    caboAccountName,
                    filePath);
        }
        else
        {
            dataLakeStoreFileSystemManagementClient.getFileSystemOperations().create(
                    caboAccountName,
                    filePath,
                    fileContentsToAdd.getBytes(),
                    true);
        }

        return filePath;
    }

    private FileStatusResult GetAndCompareFileOrFolder(String caboAccountName, String fileOrFolderPath, FileType expectedType, long expectedLength) throws Exception
    {
        FileStatusResult getResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().getFileStatus(caboAccountName, fileOrFolderPath).getBody();
        Assert.assertEquals(expectedLength, (long) getResponse.getFileStatus().getLength());
        Assert.assertEquals(expectedType, getResponse.getFileStatus().getType());

        return getResponse;
    }

    private void CompareFileContents(String caboAccountName, String filePath, String expectedContents) throws Exception
    {
        // download a file and ensure they are equal
        InputStream openResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().open(caboAccountName, filePath).getBody();
        Assert.assertNotNull(openResponse);
        Writer writer = new StringWriter();
        char[] buffer = new char[1024];
        try {
            Reader reader = new BufferedReader(
                    new InputStreamReader(openResponse, "UTF-8"));
            int n;
            while ((n = reader.read(buffer)) != -1) {
                writer.write(buffer, 0, n);
            }
        }
        finally {
            openResponse.close();
        }
        String fileContents =  writer.toString();
        Assert.assertEquals(expectedContents, fileContents);
    }

    private void DeleteFolder(String caboAccountName, String folderPath, boolean recursive, boolean failureExpected) throws Exception
    {
        if (failureExpected)
        {
            // try to delete a folder that doesn't exist or should fail
            try
            {
                FileOperationResult deleteFolderResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().delete(caboAccountName, folderPath, recursive).getBody();
                Assert.assertTrue(!deleteFolderResponse.getOperationResult());
            }
            catch (Exception e)
            {
                Assert.assertTrue(e instanceof CloudException);
            }
        }
        else
        {
            // Delete a folder
            FileOperationResult deleteFolderResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().delete(caboAccountName, folderPath, recursive).getBody();
            Assert.assertTrue(deleteFolderResponse.getOperationResult());
        }
    }

    private void DeleteFile(String caboAccountName, String filePath, boolean failureExpected) throws Exception
    {
        if (failureExpected)
        {
            // try to delete a file that doesn't exist
            try
            {
                FileOperationResult deleteFileResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().delete(caboAccountName, filePath, false).getBody();
                Assert.assertTrue(!deleteFileResponse.getOperationResult());
            }
            catch (Exception e)
            {
                Assert.assertTrue(e instanceof CloudException);
            }
        }
        else
        {
            // Delete a file
            FileOperationResult deleteFileResponse = dataLakeStoreFileSystemManagementClient.getFileSystemOperations().delete(caboAccountName, filePath, false).getBody();
            Assert.assertTrue(deleteFileResponse.getOperationResult());
        }
    }
}
