/**
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 */

package com.microsoft.azure.management.website.models;

import org.joda.time.DateTime;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Represents a recommendation result generated by the recommendation engine.
 */
public class Recommendation {
    /**
     * Timestamp when this instance was created.
     */
    private DateTime creationTime;

    /**
     * A GUID value that each recommendation object is associated with.
     */
    private String recommendationId;

    /**
     * Full ARM resource ID string that this recommendation object is
     * associated with.
     */
    private String resourceId;

    /**
     * Name of a resource type this recommendation applies, e.g. Subscription,
     * ServerFarm, Site.
     */
    private String resourceScope;

    /**
     * Unique name of the rule.
     */
    private String ruleName;

    /**
     * UI friendly name of the rule (may not be unique).
     */
    private String displayName;

    /**
     * Localized text of recommendation, good for UI.
     */
    private String message;

    /**
     * Level indicating how critical this recommendation can impact. Possible
     * values include: 'Critical', 'Warning', 'Information',
     * 'NonUrgentSuggestion'.
     */
    @JsonProperty(required = true)
    private NotificationLevel level;

    /**
     * List of channels that this recommendation can apply. Possible values
     * include: 'Notification', 'Api', 'Email', 'All'.
     */
    @JsonProperty(required = true)
    private Channels channels;

    /**
     * The category that this recommendation belongs to. Possible values
     * include: 'Uncategorized', 'Test', 'UpSell', 'CrossSell', 'LiveSite'.
     */
    @JsonProperty(required = true)
    private Category category;

    /**
     * Name of action recommended by this object.
     */
    private String actionName;

    /**
     * On/off flag indicating the rule is currently enabled or disabled.
     */
    private Integer enabled;

    /**
     * The beginning time of a range that the recommendation refers to.
     */
    private DateTime startTime;

    /**
     * The end time of a range that the recommendation refers to.
     */
    private DateTime endTime;

    /**
     * When to notify this recommendation next. Null means that this will
     * never be notified anymore.
     */
    private DateTime nextNotificationTime;

    /**
     * Date and time when this notification expires.
     */
    private DateTime notificationExpirationTime;

    /**
     * Last timestamp this instance was actually notified. Null means that
     * this recommendation hasn't been notified yet.
     */
    private DateTime notifiedTime;

    /**
     * A metric value measured by the rule.
     */
    private Double score;

    /**
     * Get the creationTime value.
     *
     * @return the creationTime value
     */
    public DateTime getCreationTime() {
        return this.creationTime;
    }

    /**
     * Set the creationTime value.
     *
     * @param creationTime the creationTime value to set
     */
    public void setCreationTime(DateTime creationTime) {
        this.creationTime = creationTime;
    }

    /**
     * Get the recommendationId value.
     *
     * @return the recommendationId value
     */
    public String getRecommendationId() {
        return this.recommendationId;
    }

    /**
     * Set the recommendationId value.
     *
     * @param recommendationId the recommendationId value to set
     */
    public void setRecommendationId(String recommendationId) {
        this.recommendationId = recommendationId;
    }

    /**
     * Get the resourceId value.
     *
     * @return the resourceId value
     */
    public String getResourceId() {
        return this.resourceId;
    }

    /**
     * Set the resourceId value.
     *
     * @param resourceId the resourceId value to set
     */
    public void setResourceId(String resourceId) {
        this.resourceId = resourceId;
    }

    /**
     * Get the resourceScope value.
     *
     * @return the resourceScope value
     */
    public String getResourceScope() {
        return this.resourceScope;
    }

    /**
     * Set the resourceScope value.
     *
     * @param resourceScope the resourceScope value to set
     */
    public void setResourceScope(String resourceScope) {
        this.resourceScope = resourceScope;
    }

    /**
     * Get the ruleName value.
     *
     * @return the ruleName value
     */
    public String getRuleName() {
        return this.ruleName;
    }

    /**
     * Set the ruleName value.
     *
     * @param ruleName the ruleName value to set
     */
    public void setRuleName(String ruleName) {
        this.ruleName = ruleName;
    }

    /**
     * Get the displayName value.
     *
     * @return the displayName value
     */
    public String getDisplayName() {
        return this.displayName;
    }

    /**
     * Set the displayName value.
     *
     * @param displayName the displayName value to set
     */
    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    /**
     * Get the message value.
     *
     * @return the message value
     */
    public String getMessage() {
        return this.message;
    }

    /**
     * Set the message value.
     *
     * @param message the message value to set
     */
    public void setMessage(String message) {
        this.message = message;
    }

    /**
     * Get the level value.
     *
     * @return the level value
     */
    public NotificationLevel getLevel() {
        return this.level;
    }

    /**
     * Set the level value.
     *
     * @param level the level value to set
     */
    public void setLevel(NotificationLevel level) {
        this.level = level;
    }

    /**
     * Get the channels value.
     *
     * @return the channels value
     */
    public Channels getChannels() {
        return this.channels;
    }

    /**
     * Set the channels value.
     *
     * @param channels the channels value to set
     */
    public void setChannels(Channels channels) {
        this.channels = channels;
    }

    /**
     * Get the category value.
     *
     * @return the category value
     */
    public Category getCategory() {
        return this.category;
    }

    /**
     * Set the category value.
     *
     * @param category the category value to set
     */
    public void setCategory(Category category) {
        this.category = category;
    }

    /**
     * Get the actionName value.
     *
     * @return the actionName value
     */
    public String getActionName() {
        return this.actionName;
    }

    /**
     * Set the actionName value.
     *
     * @param actionName the actionName value to set
     */
    public void setActionName(String actionName) {
        this.actionName = actionName;
    }

    /**
     * Get the enabled value.
     *
     * @return the enabled value
     */
    public Integer getEnabled() {
        return this.enabled;
    }

    /**
     * Set the enabled value.
     *
     * @param enabled the enabled value to set
     */
    public void setEnabled(Integer enabled) {
        this.enabled = enabled;
    }

    /**
     * Get the startTime value.
     *
     * @return the startTime value
     */
    public DateTime getStartTime() {
        return this.startTime;
    }

    /**
     * Set the startTime value.
     *
     * @param startTime the startTime value to set
     */
    public void setStartTime(DateTime startTime) {
        this.startTime = startTime;
    }

    /**
     * Get the endTime value.
     *
     * @return the endTime value
     */
    public DateTime getEndTime() {
        return this.endTime;
    }

    /**
     * Set the endTime value.
     *
     * @param endTime the endTime value to set
     */
    public void setEndTime(DateTime endTime) {
        this.endTime = endTime;
    }

    /**
     * Get the nextNotificationTime value.
     *
     * @return the nextNotificationTime value
     */
    public DateTime getNextNotificationTime() {
        return this.nextNotificationTime;
    }

    /**
     * Set the nextNotificationTime value.
     *
     * @param nextNotificationTime the nextNotificationTime value to set
     */
    public void setNextNotificationTime(DateTime nextNotificationTime) {
        this.nextNotificationTime = nextNotificationTime;
    }

    /**
     * Get the notificationExpirationTime value.
     *
     * @return the notificationExpirationTime value
     */
    public DateTime getNotificationExpirationTime() {
        return this.notificationExpirationTime;
    }

    /**
     * Set the notificationExpirationTime value.
     *
     * @param notificationExpirationTime the notificationExpirationTime value to set
     */
    public void setNotificationExpirationTime(DateTime notificationExpirationTime) {
        this.notificationExpirationTime = notificationExpirationTime;
    }

    /**
     * Get the notifiedTime value.
     *
     * @return the notifiedTime value
     */
    public DateTime getNotifiedTime() {
        return this.notifiedTime;
    }

    /**
     * Set the notifiedTime value.
     *
     * @param notifiedTime the notifiedTime value to set
     */
    public void setNotifiedTime(DateTime notifiedTime) {
        this.notifiedTime = notifiedTime;
    }

    /**
     * Get the score value.
     *
     * @return the score value
     */
    public Double getScore() {
        return this.score;
    }

    /**
     * Set the score value.
     *
     * @param score the score value to set
     */
    public void setScore(Double score) {
        this.score = score;
    }

}
